@@ -5,38 +5,36 @@ use Carp;
 use File::Spec;
 use File::Path;
 use version;
+use YAML::Tiny qw( LoadFile DumpFile );
+use Cwd;
 
 use Test::Database::Handle;
 
 #
-# global configuration
+# GLOBAL CONFIGURATION
 #
+
 # the location where all drivers-related files will be stored
 my $root
     = File::Spec->rel2abs(
     File::Spec->catdir( File::Spec->tmpdir(), 'Test-Database-' . getlogin() )
     );
 
-# some information stores, indexed by driver class name
-my %drh;
-
 # generic driver class initialisation
 sub __init {
     my ($class) = @_;
 
     # create directory if needed
-    if ( $class->is_filebased() ) {
-        my $dir = $class->base_dir();
-        if ( !-e $dir ) {
-            mkpath( [$dir] );
-        }
-        elsif ( !-d $dir ) {
-            croak "$dir is not a directory. Initializing $class failed";
-        }
+    my $dir = $class->base_dir();
+    if ( !-e $dir ) {
+        mkpath( [$dir] );
+    }
+    elsif ( !-d $dir ) {
+        croak "$dir is not a directory. Initializing $class failed";
     }
 
-    # load the DBI driver
-    $drh{ $class->name() } ||= DBI->install_driver( $class->name() );
+    # load the DBI driver (may die)
+    DBI->install_driver( $class->name() );
 }
 
 #
@@ -46,37 +44,34 @@ sub new {
     my ( $class, %args ) = @_;
 
     if ( $class eq __PACKAGE__ ) {
-        croak "No driver defined" if !exists $args{driver};
-        eval "require Test::Database::Driver::$args{driver}"
-            or croak $@;
-        $class = "Test::Database::Driver::$args{driver}";
-        $class->__init();    # survive a cleanup()
+        croak "dbd parameter required" if !exists $args{dbd};
+        eval "require Test::Database::Driver::$args{dbd}"
+            or do { $@ =~ s/ at .*?\z//s; croak $@; };
+        $class = "Test::Database::Driver::$args{dbd}";
+        $class->__init();
     }
-    bless {
+
+    my $self = bless {
         username => '',
         password => '',
-        map ( { $_ => '' } $class->essentials() ),
         %args,
-        driver => $class->name()
+        dbd => $class->name() || $args{dbd},
         },
         $class;
+
+    $self->_load_mapping();
+
+    # try to connect before returning the object
+    if ( !$class->is_filebased() ) {
+        eval { DBI->connect_cached( $self->connection_info() ) }
+            or return;
+    }
+
+    return $self;
 }
 
-sub cleanup {
-    my ($self) = @_;
-    if ( $self->is_filebased() ) {
-        my $dir = $self->base_dir();
-        for my $entry ( map { File::Spec->catfile( $dir, $_ ) }
-            $self->_filebased_databases() )
-        {
-            if ( -d $entry ) {
-                rmtree( [$entry] );
-            }
-            else {
-                unlink $entry;
-            }
-        }
-    }
+sub _mapping_file {
+    return File::Spec->catfile( $_[0]->base_dir(), 'mapping.yml' );
 }
 
 sub available_dbname {
@@ -88,50 +83,70 @@ sub available_dbname {
     return "$name$n";
 }
 
-sub as_string {
-    return join '',
-        map { "$_ = " . _quote( $_[0]{$_} || '' ) . "\n" }
-        driver => $_[0]->essentials();
+sub _load_mapping {
+    my ($self, $file)= @_;
+    $file = $self->_mapping_file() if ! defined $file;
+
+    # basic mapping info
+    $self->{mapping} = {};
+    return if !-e $file;
+
+    # load mapping from file
+    my $mapping = LoadFile( $file );
+    $self->{mapping} = $mapping->{$self->driver_dsn()} || {};
 }
 
-sub handles {
-    my ( $self, @requests ) = @_;
+sub _save_mapping {
+    my ($self, $file )= @_;
+    $file = $self->_mapping_file() if ! defined $file;
 
-    # return all available handles if no request
-    my @databases = $self->databases();
-    return map { $self->_handle($_) } @databases if !@requests;
+    # update mapping information
+    my $mapping = {};
+    $mapping = LoadFile( $file ) if -e $file;
+    $mapping->{ $self->driver_dsn() } = $self->{mapping};
 
-    # get unique names, with higher priority on keep
-    # '' will get a random name
-    my %keep;
-    for my $request (@requests) {
-        my $dbname = exists $request->{name} ? $request->{name} : '';
-        $keep{$dbname} ||= $request->{keep};
-    }
-
-    # create all databases if needed
-    return map { $self->create_database( $_, $keep{$_} ) } keys %keep;
+    # save mapping information
+    DumpFile( "$file.tmp", $mapping );
+    rename "$file.tmp", $file
+        or croak "Can't rename $file.tmp to $file: $!";
 }
 
-my @DROP;
-sub register_drop { push @DROP, [@_]; }
+sub make_handle {
+    my ($self) = @_;
+    my $handle;
 
-END {
-    for my $drop (@DROP) {
-        my ( $driver, $dbname ) = @$drop;
-        $driver->drop_database($dbname);
+    # return a handle for this driver, using the mapping
+    if ( my $dbname = $self->{mapping}{ cwd() } ) {
+        $handle = Test::Database::Handle->new(
+            dsn    => $self->dsn($dbname),
+            name   => $dbname,
+            driver => $self,
+        );
     }
+
+    # otherwise create the database and update the mapper
+    else {
+        $handle = $self->create_database();
+        $self->{mapping}{ cwd() } = $handle->{name};
+        $self->_save_mapping();
+    }
+
+    return $handle;
 }
 
 #
 # ACCESSORS
 #
 sub name { return ( $_[0] =~ /^Test::Database::Driver::([:\w]*)/g )[0]; }
+*dbd = \&name;
 
 sub base_dir {
-    return $_[0] eq __PACKAGE__
-        ? $root
-        : File::Spec->catdir( $root, $_[0]->name() );
+    my ($self) = @_;
+    my $class = ref $self || $self;
+    return $root if $class eq __PACKAGE__;
+    my $dir = File::Spec->catdir( $root, $class->name() );
+    return $dir if !ref $self;    # class method
+    return $self->{base_dir} ||= $dir;    # may be overriden in new()
 }
 
 sub version {
@@ -139,13 +154,12 @@ sub version {
     return $_[0]{version} ||= version->new( $_[0]->_version() );
 }
 
-sub drh      { return $drh{ $_[0]->name() } }
-sub bare_dsn { return $_[0]{dsn} ||= $_[0]->_bare_dsn() }
+sub driver_dsn { return $_[0]{dsn} ||= $_[0]->_driver_dsn() }
 sub username { return $_[0]{username} }
 sub password { return $_[0]{password} }
 
 sub connection_info {
-    return ( $_[0]->bare_dsn(), $_[0]->username(), $_[0]->password() );
+    return ( $_[0]->driver_dsn(), $_[0]->username(), $_[0]->password() );
 }
 
 # THESE MUST BE IMPLEMENTED IN THE DERIVED CLASSES
@@ -153,9 +167,11 @@ sub drop_database { die "$_[0] doesn't have a drop_database() method\n" }
 sub _version      { die "$_[0] doesn't have a _version() method\n" }
 sub dsn           { die "$_[0] doesn't have a dsn() method\n" }
 
+# create_database creates the database and returns a handle
 sub create_database {
-    goto &_filebased_create_database if $_[0]->is_filebased();
-    die "$_[0] doesn't have a create_database() method\n";
+    my $class = ref $_[0] || $_[0];
+    goto &_filebased_create_database if $class->is_filebased();
+    die "$class doesn't have a create_database() method\n";
 }
 
 sub databases {
@@ -164,30 +180,30 @@ sub databases {
 }
 
 # THESE MAY BE OVERRIDDEN IN THE DERIVED CLASSES
-sub essentials   { }
 sub is_filebased {0}
-sub _bare_dsn    { join ':', 'dbi', $_[0]->name(), ''; }
+sub _driver_dsn    { join ':', 'dbi', $_[0]->name(), ''; }
 
 #
 # PRIVATE METHODS
 #
-sub _basename { return 'Test_Database_' . $_[0]->name() . '_' }
+sub _basename { return lc 'TDD_' . $_[0]->name() . '_' }
 
+# generic implementations for file-based drivers
 sub _filebased_databases {
-    my ($self) = @_;
-    my $dir = $self->base_dir();
+    my ($self)   = @_;
+    my $dir      = $self->base_dir();
+    my $basename = qr/^@{[$self->_basename()]}/;
 
     opendir my $dh, $dir or croak "Can't open directory $dir for reading: $!";
-    my @databases = File::Spec->no_upwards( readdir($dh) );
+    my @databases = grep {/$basename/} File::Spec->no_upwards( readdir($dh) );
     closedir $dh;
 
     return @databases;
 }
 
 sub _filebased_create_database {
-    my ( $self, $dbname, $keep ) = @_;
-    $dbname = $self->available_dbname() if !$dbname;
-    $self->register_drop($dbname) if !$keep;
+    my ( $self ) = @_;
+    my $dbname = $self->available_dbname();
 
     return Test::Database::Handle->new(
         dsn    => $self->dsn($dbname),
@@ -196,38 +212,6 @@ sub _filebased_create_database {
     );
 }
 
-sub _handle {
-    my ( $self, $name ) = @_;
-    return Test::Database::Handle->new(
-        dsn    => $self->dsn($name),
-        name   => $name,
-        driver => $self,
-    );
-}
-
-#
-# PRIVATE FUNCTIONS
-#
-sub _quote {
-    my ($string) = @_;
-    return $string if $string =~ /^\w+$/;
-
-    $string =~ s/\\/\\\\/g;
-    $string =~ s/"/\\"/g;
-    $string =~ s/\n/\\n/g;
-    return qq<"$string">;
-}
-
-sub _unquote {
-    my ($string) = @_;
-    return $string if $string !~ /\A(["']).*\1\z/s;
-
-    my $quote = chop $string;
-    $string = substr( $string, 1 );
-    $string =~ s/\\(.)/$1 eq 'n' ? "\n" : $1/eg;
-    return $string;
-}
-
 'CONNECTION';
 
 __END__
@@ -252,7 +236,7 @@ Test::Database::Driver - Base class for Test::Database drivers
     }
 
     sub create_database {
-        my ( $self, $dbname, $keep ) = @_;
+        my ( $self ) = @_;
         ...;
         return $handle;
     }
@@ -286,8 +270,19 @@ Create a new C<Test::Database::Driver> object.
 If called as C<< Test::Database::Driver->new() >>, requires a C<driver>
 parameter to define the actual object class.
 
+=item make_handle()
+
+Create a new C<Test::Database::Handle> object, attached to an existing database
+or to a newly created one.
+
+The decision whether to create a new database or not is made by
+C<Test::Database::Driver> based on the information in the mapper.
+See L<TEMPORARY STORAGE ORGANIZATION> for details.
+
 =item name()
 
+=item dbd()
+
 The driver's short name (everything after C<Test::Database::Driver::>).
 
 =item base_dir()
@@ -300,13 +295,9 @@ if needed. Typically used by file-based database drivers.
 C<version> object representing the version of the underlying database enginge.
 This object is build with the return value of C<_version()>.
 
-=item drh()
+=item driver_dsn()
 
-The DBI driver for this driver.
-
-=item bare_dsn()
-
-Return a bare Data Source Name, sufficient to connect to the database
+Return a driver Data Source Name, sufficient to connect to the database
 engine without specifying an actual database.
 
 =item username()
@@ -319,24 +310,9 @@ Return the connection password.
 
 =item connection_info()
 
-Return the connection information triplet (C<bare_dsn>, C<username>,
+Return the connection information triplet (C<driver_dsn>, C<username>,
 C<password>).
 
-=item as_string()
-
-Return a string representation of the C<Test::Database::Driver>,
-suitable to be saved in a configuration file.
-
-=item handles( @requests )
-
-Return C<Test::Database::Handler> objects matching the given requests.
-
-If no request is given, return a handler for each of the existing databases.
-
-=item cleanup()
-
-Remove the directory used by C<Test::Database> drivers.
-
 =back
 
 The class also provides a few helpful commands that may be useful for driver
@@ -353,11 +329,6 @@ for the driver.
 
 Return a bare Data Source Name, for the database with the given C<$dbname>.
 
-=item register_drop( $dbname )
-
-Register the database with the given C<$dbname> to be dropped automatically
-when the current program ends.
-
 =back
 
 =head1 WRITING A DRIVER FOR YOUR DATABASE OF CHOICE
@@ -397,21 +368,35 @@ Return a boolean value indicating if the database engine is file-based
 or not, i.e. if all the database information is stored in a file or a
 directory, and no external database server is needed.
 
-=item essentials()
-
-Return the I<essential> fields needed to serialize the driver.
-
 =item databases()
 
 Return the names of all existing databases for this driver as a list
 (the default implementation is only valid for file-based drivers).
 
-=item cleanup()
+=back
 
-Clean all databases created with names generated with C<available_dbname()>.
+=head1 TEMPORARY STORAGE ORGANIZATION
 
-For file-based databases, the directory used by the C<Test::Database::Driver>
-subclass will be deleted.
+Subclasses of C<Test::Database::Driver> store useful information
+in the system's temporary directory, under a directory named
+F<Test-Database-$user> (C<$user> being the current user's name).
+
+That directory contains the following files:
+
+=over 4
+
+=item database files
+
+The database files and directories created by file-based drivers
+controlled by C<Test::Database> are stored here, under names matching
+F<tdd_B<DRIVER>_B<N>>, where B<DRIVER> is the lowercased name of the
+driver and B<N> is a number.
+
+=item the F<mapping.yml> file
+
+A YAML file containing a C<cwd()> / database name mapping, to enable a
+given test suite to receive the same database handles in all the test
+scripts that call the C<Test::Database->handles()> method.
 
 =back
 
